@@ -20,8 +20,6 @@ import codecs
 import datetime
 from collections import deque, namedtuple
 import types
-#import imp
-#import importlib
 import inspect
 import copy
 import uuid
@@ -34,7 +32,6 @@ import pkgutil
 import itertools
 
 from .compat import *
-from .utils import String, ByteString, Mock
 from .data import (
     _names,
     _unicode_names,
@@ -48,26 +45,10 @@ from .data import (
     _last_names,
     usa,
 )
-# from .data.usa import (
-#     cities,
-#     states,
-#     zipcodes,
-# )
 
 from . import environ
-from .path import (
-    Dirpath,
-    Filepath,
-    Modulepath,
-    ContentBytes,
-    ContentString,
-    ContentFilepath,
-    CSVpath,
-)
 from .threading import Thread, Tail
 from .output import Capture
-from .server import PathServer, CookieServer, CallbackServer
-from .service import Upstart, InitD, Systemd
 from .client import Command, ModuleCommand, FileCommand, HTTP
 from .test import (
     TestCase,
@@ -81,10 +62,13 @@ from .test import (
     expected_failure,
     expect_failure,
 )
-from .image import make_png
+from .path import *
+from .service import *
+from .mock import *
+from .server import *
 
 
-__version__ = '1.4.8'
+__version__ = '4.0.0'
 
 
 # get rid of "No handler found" warnings (cribbed from requests)
@@ -198,50 +182,6 @@ change = environment
 configure = environment
 
 
-def start_service(service_name, ignore_failure=True):
-    """start a local service
-
-    :param service_name: string, the service you want to start
-    :param ignore_failure: bool, True if it should ignore a failure return code
-    :returns: Service instance
-    """
-    for service_class in [Systemd, Upstart, InitD]:
-        s = service_class(service_name)
-        s.ignore_failure = ignore_failure
-        if s.exists():
-            s.start()
-            break
-        else:
-            s = None
-
-    if s is None:
-        raise RuntimeError("Could not find a valid service for {}".format(service_name))
-
-    return s
-
-
-def stop_service(service_name, ignore_failure=True):
-    """stop a local service
-
-    :param service_name: string, the service you want to stop
-    :param ignore_failure: bool, True if it should ignore a failure return code
-    :returns: Service instance
-    """
-    for service_class in [Systemd, Upstart, InitD]:
-        s = service_class(service_name)
-        s.ignore_failure = ignore_failure
-        if s.exists():
-            s.stop()
-            break
-        else:
-            s = None
-
-    if s is None:
-        raise RuntimeError("Could not find a valid service for {}".format(service_name))
-
-    return s
-
-
 def capture(stdout=True, stderr=True, loggers=True, *args, **kwargs):
     """Capture stdout and stderr so you can inspect it
 
@@ -363,18 +303,29 @@ def wait(callback, cb_args=None, cb_kwargs=None, timeout=30.0, interval=0.1):
                 raise RuntimeError("wait() timed out after {} seconds".format(timeout))
 
 
-def choice(*args):
+def choice(*args, **kwargs):
     """Wrapper around random.choice that makes sure everything is a list, handy
     for python 3 code where you have to wrap a lot of generators in list(...)
 
     :param *args: iter(s), one or more iterators or lists that will all be combined
         into one giant list
+    :param exclude: list, a list of values that shouldn't be selected from *args
     :returns: a single object from all the *args
     """
+    exclude = kwargs.pop("exclude", None)
+    exclude = set(exclude) if exclude else set()
     vals = []
     for arg in args:
-        vals.extend(arg)
-    return random.choice(vals)
+        if isinstance(arg, basestring) or not isinstance(arg, Sequence):
+            vals.append(arg)
+        else:
+            vals.extend(arg)
+
+    ret = random.choice(vals)
+    while ret in exclude:
+        ret = random.choice(vals)
+    return ret
+choose = choice
 
 
 def get_bool():
@@ -444,436 +395,6 @@ def fetch(url, body=None, query=None, **kwargs):
     return ret
 
 
-def create_fileserver(file_dict, tmpdir="", hostname="", port=0, encoding=""):
-    """
-    create a fileserver that can be used to test remote file retrieval
-
-    :param file_dict: dict|list|str, same as create_files
-    :param tmpdir: str, same as create_files
-    :param hostname: str, usually leave this alone and it will use localhost
-    :param port: int, the port you want to use
-    """
-    if isinstance(file_dict, Sequence):
-        file_dict = {
-            "index.html": file_dict
-        }
-
-    path = create_files(file_dict, tmpdir=tmpdir, encoding=encoding)
-    return PathServer(path, hostname=hostname, port=port, encoding=encoding)
-
-
-def create_cookieserver(cookies, hostname="", port=0):
-    """
-    create a fileserver that can be used to test remote file retrieval
-
-    :param cookies: a dict of name: val or a list ot tuples(name, val)
-    :param hostname: str, usually leave this alone and it will use localhost
-    :param port: int, the port you want to use
-    """
-    return CookieServer(cookies, hostname=hostname, port=port)
-
-
-def get_content_file(fileroot, basedir="", encoding=""):
-    return ContentFilepath(fileroot, basedir=basedir, encoding=encoding)
-get_content_path = get_content_file
-get_path = get_content_file
-get_data_path = get_content_file
-
-
-def get_content_body(fileroot, basedir="", encoding=""):
-    """Returns the contents of a file matching basedir/fileroot.*
-
-    :param fileroot: string, can be a basename (fileroot.ext) or just a file root, 
-        in which case basedir/fileroot.* will be searched for and first file matched
-        will be used
-    :param basedir: string, the directory to search for fileroot.*, if not passed
-        in then os.getcwd()/*/testdata will be searched for
-    :returns: string, the contents of the found file
-    """
-    if encoding:
-        return ContentString(fileroot, basedir=basedir, encoding=encoding)
-    else:
-        return ContentBytes(fileroot, basedir=basedir)
-get_contents = get_content_body
-get_content_contents = get_content_body
-get_content_data = get_content_body
-get_data = get_content_body
-
-
-def create_dir(path="", tmpdir=""):
-    '''
-    create a directory path using a tempdir as the root
-
-    so, if you pass in "/foo/bar" that will be combined with a tempdir, so you end 
-    up with the final path: /tmp/python/dir/foo/bar
-
-    :param path: string, the temp dir path
-    :param tmpdir: string, the temp directory to use as the base
-    :returns: Dirpath, the full directory path
-    '''
-    return Dirpath.create_instance(path, tmpdir)
-create_directory = create_dir
-create_d = create_dir
-
-
-def create_dirs(dirs, tmpdir=""):
-    """
-    create a whole bunch of directories all at once
-
-    :param dirs: list, the directories to create relative to tmpdir
-    :param tmpdir: string, the base directory
-    :returns: Dirpath instance pointing to the base directory all of dirs were
-        created in
-    """
-    base_dir = Dirpath(basedir=tmpdir)
-    base_dir.create_dirs(dirs)
-    return base_dir
-create_ds = create_dirs
-
-
-def get_dir(path):
-    """
-    return a directory path
-
-    :param path: string, the path to a real directory
-    :returns: Dirpath, the path wrapped with all the Dirpath functionality
-    """
-    return Dirpath.get_instance(path)
-get_directory = get_dir
-get_d = get_dir
-
-
-def create_file(path="", contents="", tmpdir="", encoding=""):
-    '''
-    create a file and return the full path to that file
-
-    :param path: string, the path to the file
-    :param contents: string, the file contents
-    :param tmpdir: string, the temp directory to use as the base
-    :param encoding: string, whatever encoding you want the file to have
-
-    :returns: Filepath, the full file path
-    '''
-    if not path:
-        path = get_module_name(prefix="cf")
-
-    instance = Filepath.create_instance(path, contents, tmpdir, encoding)
-
-    if encoding:
-        instance = Filepath(path, tmpdir)
-        instance.encoding = encoding
-        instance.create(contents)
-    else:
-        instance = Filepath.create_instance(path, contents, tmpdir, encoding)
-    return instance
-create_f = create_file
-
-
-def create_files(file_dict, tmpdir="", encoding=""):
-    """
-    create a whole bunch of files all at once
-
-    file_dict -- dict -- keys are the filepath relative to tmpdir, values are the
-        file contents
-    tmpdir -- Dirpath -- same as create_module() tmpdir
-    """
-    base_dir = Dirpath(basedir=tmpdir)
-    base_dir.create_files(file_dict, encoding)
-    return base_dir
-create_fs = create_files
-
-
-def get_file(path="", tmpdir="", encoding=""):
-    if not path:
-        path = get_ascii()
-
-    return Filepath(path, tmpdir, encoding)
-get_f = get_file
-
-
-def create_csv(columns, count=0, path="", tmpdir="", encoding="", header=True, **kwargs):
-    """Create a csv file using the generators/callbacks found in columns
-
-
-    :param columns: dict, in the format of key: callback where the callback can
-        generate a value for the row, so something like "foo": testdata.get_name
-        would work
-    :param count: int, how many rows you want, will be randomly created between 
-        1 and 50 if not specified
-    :param path: string, the path relative to tmpdir, default is randomly generated
-    :param tmpdir: string, the temp directory to use as a base/prefix
-    :param encoding: string, the encoding to use for the csv file
-    :param **kwargs: dict, these will get passed to csv.DictWriter,
-        https://docs.python.org/3/library/csv.html#csv.DictWriter
-    :returns: testdata.path.CSVpath instance
-    """
-    if not count:
-        count = random.randint(1, 50)
-
-    if not path:
-        path = get_filename(ext="csv")
-
-    filepath = CSVpath(
-        relpath=path,
-        basedir=tmpdir,
-        encoding=encoding,
-        header=header,
-        **kwargs
-    )
-
-    rows = []
-    for i in range(count):
-        d = {}
-        for field_name, callback in columns.items():
-            if callable(callback):
-                d[field_name] = callback()
-            else:
-                d[field_name] = callback
-
-        rows.append(d)
-
-    filepath.create(rows)
-    return filepath
-
-
-def create_image(image_type="", path="", tmpdir=""):
-    """Creates an image using the images founc in the data/ directory
-
-    :param image_type: string, the type of image you want, one of jpg, png, gif, agif, ico
-    :param path: string, the path or basename (eg, foo/bar.jpg or che) of the image
-    :param tmpdir: Dirpath, same as create_module() tmpdir
-    :returns: Filepath, the path to the image file
-    """
-    images = [
-        (set(["jpg", "jpeg"]), ".jpg", "static.jpg"),
-        (set(["png"]), ".png", "static.png"),
-        (set(["gif"]), ".gif", "static.gif"),
-        (set(["agif", "animated_gif"]), ".gif", "animated.gif"),
-        (set(["ico", "favicon", "ico"]), ".ico", "favicon.ico"),
-    ]
-
-    if image_type:
-        image_type = image_type.lower()
-        for itypes, ext, image in images:
-            if image_type in itypes:
-                break
-
-    else:
-        image_type, ext, image = random.choice(images)
-
-    # https://docs.python.org/2/library/pkgutil.html#pkgutil.get_data
-    contents = pkgutil.get_data(__name__.split(".")[0], "data/{}".format(image))
-
-    if path:
-        if not path.lower().endswith(ext):
-            path += ext
-    else:
-        path = get_filename(ext=ext)
-
-    return create_file(
-        path=path,
-        contents=contents,
-        tmpdir=tmpdir,
-        encoding=None
-    )
-
-
-def create_jpg(path="", tmpdir=""):
-    """create a jpeg image"""
-    return create_image(image_type="jpg", path=path, tmpdir=tmpdir)
-create_jpeg=create_jpg
-
-
-def create_png(path="", tmpdir="", width=0, height=0, color=None):
-    """create a png image"""
-    if width and height:
-        ext = ".png"
-        if path:
-            if not path.lower().endswith(ext):
-                path += ext
-        else:
-            path = get_filename(ext=ext)
-
-        return create_file(
-            path=path,
-            contents=make_png(width, height, color=color),
-            tmpdir=tmpdir,
-            encoding=None
-        )
-
-    else:
-        return create_image(image_type="png", path=path, tmpdir=tmpdir)
-
-
-def create_gif(path="", tmpdir=""):
-    """create a static gif image"""
-    return create_image(image_type="gif", path=path, tmpdir=tmpdir)
-
-
-def create_animated_gif(path="", tmpdir=""):
-    """create an animated gif image"""
-    return create_image(image_type="agif", path=path, tmpdir=tmpdir)
-create_agif=create_animated_gif
-
-
-def create_ico(path="", tmpdir=""):
-    """create an icon image"""
-    return create_image(image_type="ico", path=path, tmpdir=tmpdir)
-create_icon=create_ico
-create_favicon=create_ico
-
-
-def get_filename(ext="", prefix="", name=""):
-    """return just a valid file name
-
-    :param ext: string, the extension you want the file to have
-    :param prefix: string, this will be the first part of the file's name
-    :param name: string, the name you want to use (prefix will be added to the front
-        of the name and ext will be added to the end of the name)
-    :returns: string, the random filename
-    """
-    if ext:
-        if not ext.startswith("."):
-            ext = "." + ext
-
-        if name:
-            if name.endswith(ext):
-                ext = ""
-
-    return get_module_name(prefix=prefix, postfix=ext, name=name)
-get_file_name = get_filename
-filename = get_filename
-file_name = get_filename
-
-
-def get_module_name(bits=1, prefix="", postfix="", name=""):
-    """Returns just a valid module name or module path
-
-    :param bits: int, how many parts you want in your module path (1 is foo, 2 is foo.bar, etc)
-    :param prefix: string, if you want the last bit to be prefixed with something
-    :param postfix: string, if you want the last bit to be posfixed with something (eg, ".py")
-    :param name: string, the name you want to use for the last bit
-        (prefix will be added to the front of the name and postfix will be added to
-         the end of the name)
-    :returns: string, the modulepath
-    """
-    parts = []
-    bits = max(bits, 1)
-
-    for x in range(bits):
-        parts.append(get_str(str_size=8, chars=string.ascii_letters).lower())
-
-    if name:
-        parts[-1] = name
-
-    if prefix:
-        parts[-1] = "{}{}".format(prefix, parts[-1])
-
-    if postfix:
-        parts[-1] = "{}{}".format(parts[-1], postfix)
-
-    return ".".join(parts)
-get_package_name = get_module_name
-get_modulename = get_module_name
-get_modname = get_module_name
-get_modpath = get_module_name
-get_modulepath = get_module_name
-get_module_path = get_module_name
-modulename = get_module_name
-module_name = get_module_name
-
-
-def get_classname(name=""):
-    n = get_filename(name=name)
-    return n.title()
-get_class_name = get_classname
-classname = get_classname
-class_name = get_classname
-
-
-def get_source_filepath(v):
-    """Returns the full filepath of a given python object
-
-    :param v: mixed, a python object to find the source file for
-    :returns: Filepath
-    """
-    try:
-        ret = inspect.getsourcefile(v)
-    except TypeError:
-        ret = inspect.getsourcefile(v.__class__)
-    return Filepath(ret)
-get_source_file = get_source_filepath
-get_source_path = get_source_filepath
-
-
-def create_module(module_name="", contents="", tmpdir="", make_importable=True):
-    '''
-    create a python module folder structure so that the module can be imported
-
-    module_name -- string -- something like foo.bar
-    contents -- string -- the contents of the module
-    tmpdir -- string -- the temp directory that will be added to the syspath if make_importable is True
-    make_importable -- boolean -- if True, then tmpdir will be added to the python path so it can be imported
-
-    return -- Module -- the module file path
-    '''
-    if not module_name:
-        module_name = get_module_name(prefix="cm")
-    return Modulepath.create_instance(module_name, contents, tmpdir, make_importable)
-
-
-def create_modules(module_dict, tmpdir="", make_importable=True, prefix=""):
-    """
-    create a whole bunch of modules all at once
-
-    :param module_dict: dict, keys are the module_name, values are the module contents
-    :param tmpdir: string, same as create_module() tmpdir
-    :param make_importable: boolean, same as create_module() tmpdir
-    :param prefix: string, if you want all the modules in module_dict to have a prefix, you
-        can pass this in, so if you did prefix is "foo.bar" then all the keys in module_dict
-        will be prepended with "foo.bar"
-    :returns: Dirpath
-    """
-    module_base_dir = Dirpath(basedir=tmpdir)
-
-    if prefix:
-        ks = list(module_dict.keys()) # we cast to list to make sure ks doesn't grow
-        for k in ks:
-            module_dict[".".join(filter(None, [prefix, k]))] = module_dict.pop(k)
-
-    for module_name, contents in module_dict.items():
-        Modulepath.create_instance(module_name, contents, module_base_dir, make_importable)
-        make_importable = False
-
-    return module_base_dir
-
-
-def create_package(module_name="", contents="", tmpdir="", make_importable=True):
-    '''
-    create a python package folder structure so that the package can be imported
-
-    a package is different than a module in that it is a module_name folder with
-    an __init__.py instead of module_name.py
-
-    module_name -- string -- something like foo.bar
-    contents -- string -- the contents of the module
-    tmpdir -- string -- the temp directory that will be added to the syspath if make_importable is True
-    make_importable -- boolean -- if True, then tmpdir will be added to the python path so it can be imported
-
-    return -- Module -- the module file path
-    '''
-    if not module_name:
-        module_name = get_module_name(prefix="cp")
-
-    return Modulepath.create_instance(
-        module_name,
-        contents,
-        tmpdir,
-        make_importable,
-        is_package=True
-    )
-
-
 def get_url():
     '''
     get a url, this is just a nice shortcut method to something I seemed to do a lot
@@ -1195,6 +716,66 @@ def get_counter(start=1, step=1):
     return lambda: next(counter)
 
 
+def get_range(max_size=10):
+    """Because sometimes you just want a random range
+
+    https://github.com/Jaymon/testdata/issues/74
+
+    :param max_size: int, the max range stop value you want
+    :returns: range that can be iterated
+    """
+    if yes():
+        start = 1
+        stop = get_int(1, max_size + 1)
+
+    else:
+        start = 0
+        stop = get_int(max_size=max_size)
+
+    return range(start, stop)
+
+
+def get_list(callback, max_size=100):
+    """Create a list filled with values returned from callback
+
+    https://github.com/Jaymon/testdata/issues/73
+
+    :param callback: callable, each item in the list will be populated by calling this
+    :param max_size: int, the maximum size of the list
+    :returns: list, the randomly generated list
+    """
+    ret = []
+    for x in get_range(max_size):
+        ret.append(callback())
+    return ret
+
+
+def get_dict(*keys, **kv):
+    """Create a dict filled with key/values returned from kv
+
+    https://github.com/Jaymon/testdata/issues/73
+
+    :param kv: dict, each key/callable will be used to generate a random dict key/val
+    :returns: dict, the randomly generated dict
+    """
+    if keys:
+        kv = {}
+        for k in keys:
+            kv[k] = (lambda: get_words(5)) if yes() else get_int
+
+    if not kv:
+        kv = {}
+        for x in get_range(5):
+            k = get_ascii_string()
+            v = (lambda: get_words(5)) if yes() else get_int
+            kv[k] = v
+
+    ret = {}
+    for k, callback in kv.items():
+        ret[k] = callback()
+    return ret
+
+
 def get_ascii_words(count=0, as_str=True, **kwargs):
     return get_words(count, as_str, words=_ascii_words, **kwargs)
 
@@ -1715,213 +1296,59 @@ def get_coordinate(v1, v2, round_to=7):
 get_coord = get_coordinate
 
 
-def patch_instance(mod, patches=None, **kwargs_patches):
-    if not mod:
-        raise ValueError("mod is empty")
+def get_datetime(now=None, backward=False, **kwargs):
+    """get a datetime
 
-    if not patches: patches = {}
-    patches.update(kwargs_patches) # combine both dicts
-
-    for name, patch in patches.items():
-        val = getattr(mod, name, None)
-        if inspect.isroutine(val):
-            setattr(mod, name, types.MethodType(patch, mod))
-
-        else:
-            if name in mod.__class__.__dict__ and inspect.isdatadescriptor(mod.__class__.__dict__[name]):
-
-                # http://stackoverflow.com/questions/30342212/override-attribute-access-precedence-having-a-data-descriptor/30578922#30578922
-                monkey_class = patch_class(
-                    mod.__class__,
-                    **{name: patch}
-                )
-                mod.__class__ = monkey_class
-
-            else:
-                setattr(mod, name, patch)
-
-    return mod
-
-
-def patch_class(mod, patches=None, **kwargs_patches):
-    if not mod:
-        raise ValueError("mod is empty")
-
-    if not patches: patches = {}
-    patches.update(kwargs_patches) # combine both dicts
-
-    def copy_dict(mod):
-        d = {}
-        for k, v in mod.__dict__.items():
-            if k.startswith("__"):
-                d[k] = v
-            else:
-                if inspect.isroutine(v):
-                    d[k] = v
-
-                elif inspect.isdatadescriptor(v):
-                    d[k] = v
-
-                else:
-                    d[k] = copy.deepcopy(v)
-        return d
-
-    class_name = ""
-    if is_py2:
-        class_name = b'{}Patched'.format(mod.__name__)
-    elif is_py3:
-        class_name = '{}Patched'.format(mod.__name__)
-
-    # http://stackoverflow.com/questions/9541025/how-to-copy-a-python-class
-    mod_patched = type(
-        class_name,
-        mod.__bases__,
-        #{k: copy.deepcopy(v) for k, v in mod.__dict__.items()}
-        copy_dict(mod)
-    )
-    for name, patch in patches.items():
-        setattr(mod_patched, name, patch)
-
-    return mod_patched
-
-
-def patch_module(mod, patches=None, **kwargs_patches):
-    if not mod:
-        raise ValueError("mod is empty")
-
-    if not patches: patches = {}
-    patches.update(kwargs_patches) # combine both dicts
-
-    deferred_patches = []
-    patched_modules = {}
-
-    for name, patch in patches.items():
-        if '.' in name:
-            # when a module is imported like this, import foo.bar then it is placed
-            # as an attribute to the parent module: getattr(getattr(mod, 'foo'), 'bar')
-            # this should be useful in eventually supporting this, you can work down the sub
-            # attributes and newly import them and patch
-            raise RuntimeError("nested modules like {} are not currently supported".format(name))
-
-        else:
-            deferred_patches.append((name, patch))
-
-    # now we need to find the full module path so we can reload it
-    if inspect.ismodule(mod):
-        mod_name = mod.__name__
-        mpath = inspect.getsourcefile(mod)
-        mfile = mpath
-
-    else:
-        mod_name = mod
-
-        # http://stackoverflow.com/questions/4907054/
-        def find_mod_path(p):
-            if '.' in p:
-                p, m = p.rsplit('.', 1)
-                imod_path = find_mod_path(p)
-                mod_path = os.path.join(imod_path, m)
-                #mod = imp.load_module('{}_{}'.format(m, get_ascii(8)), *imp.find_module(m, imod.__path__))
-
-            else:
-                # we fudge the paths a bit to make sure current working directory is
-                # also checked
-                paths = [os.getcwd()]
-                paths.extend(sys.path)
-                # imp is deprecated in python 3, see:
-                # https://stackoverflow.com/questions/35288021/what-is-the-equivalent-of-imp-find-module-in-importlib
-                if is_py2:
-                    _, mod_path, _ = imp.find_module(p, paths)
-                else:
-                    spec = importlib.machinery.PathFinder().find_spec(p, paths)
-                    mod_path = spec.submodule_search_locations[0]
-
-            return mod_path
-
-        mpath = find_mod_path(mod_name)
-
-        mfile = mpath
-        # figure out if we have a package or a module and set the appropriate file
-        if os.path.isdir(mpath):
-            mfile = os.path.join(mpath, '__init__.py')
-
-        else:
-            if not mfile.endswith(".py"):
-                mfile = '{}.py'.format(mpath)
-
-    mname = '{}_{}'.format(mod_name, get_ascii(8))
-    if is_py2:
-        m = imp.load_source(mname, mfile)
-    else:
-        # ugh, this is deprecated in 3.4 (though it isn't throwing a warning
-        m = importlib.machinery.SourceFileLoader(mname, mfile).load_module()
-        # https://docs.python.org/3/library/importlib.html#importing-a-source-file-directly
-
-#         loader = importlib.machinery.SourceFileLoader(mname, mfile)
-#         loaded = types.ModuleType(loader.name)
-#         m = loader.exec_module(loaded)
-
-    # go through and apply all the patches
-    for patch_name, patch in deferred_patches:
-        setattr(m, patch_name, patch)
-
-    return m
-
-
-def patch(mod, patches=None, **kwargs_patches):
-    '''
-    import module_name and apply the patches to it
-
-    mod -- string|module|class|instance -- the name of the thing you are patching
-    patches -- dict -- the keys are functions, classes, or modules that should be
-    patched in the module, the value is the patched value you want to replace the
-    key with
-
-    return -- module|object -- if you pass in an object, return the object, otherwise
-        return the module
-    '''
-    if isinstance(mod, basestring):
-        m = patch_module(mod, patches=patches, **kwargs_patches)
-
-    elif inspect.ismodule(mod):
-        m = patch_module(mod, patches=patches, **kwargs_patches)
-
-    elif inspect.isclass(mod):
-        m = patch_class(mod, patches=patches, **kwargs_patches)
-
-    else:
-        m = patch_instance(mod, patches=patches, **kwargs_patches) 
-
-    return m
-
-
-def mock_class(name="", **props_and_methods):
-    """create a class with the given method and properties
-
-    :param name: string, the name of the class, default is just a random classname
-    :param **props_and_methods: dict, keys will be attributes on the object
-    :returns: type, the object
+    :param now: datetime|date|int|float|timedelta
+        datetime - just returned
+        date - returned as a datetime
+        int - assumed to be days and will be added/subtracted from utc now
+        float - assumed to be seconds and will be added/subtracted from utc now
+        timedelta - will be add/subtracted from utc now
+    :param backward: if True, then a positive int, float, or timedelta will be
+        subtracted from utc now, if False then it will be added
+    :param **kwargs: these will be passed to timedelta
+    :returns: datetime
     """
-    classname = get_classname(name=name)
-    return type(ByteString(classname), (object,), props_and_methods)
+    if not now and kwargs:
+        now = datetime.timedelta(**kwargs)
 
+    if now:
+        if isinstance(now, datetime.datetime):
+            pass
 
-def mock_instance(name="", **props_and_methods):
-    """This is the same as mock_class but returns an instance of that class
+        elif isinstance(now, datetime.timedelta):
+            seconds = now.total_seconds()
+            microseconds = (seconds * 1000000) % 1000000
+            if backward and seconds > 0:
+                seconds *= -1.0
 
-    see mock_class() docs"""
-    return mock_class(name, **props_and_methods)()
+            td = datetime.timedelta(seconds=seconds, microseconds=microseconds)
+            now = datetime.datetime.utcnow() + td
 
+        elif isinstance(now, datetime.date):
+            now = datetime.datetime(now.year, now.month, now.day)
 
-def mock(**props_and_methods):
-    """Create a mocked object that tries to be really magical
+        elif isinstance(now, int):
+            if backward and now > 0:
+                now *= -1.0
+            now = datetime.datetime.utcnow() + datetime.timedelta(days=now)
 
-    This is different than mock_instance because it creates an object that uses a lot
-    of magic to try and be a jack of all trades
+        elif isinstance(now, float):
+            seconds = int(now)
+            microseconds = (now * 1000000) % 1000000
+            if backward and seconds > 0:
+                seconds *= -1.0
+            td = datetime.timedelta(seconds=now, microseconds=microseconds)
+            now = datetime.datetime.utcnow() + td
 
-    :returns: Mock instance
-    """
-    return Mock(**props_and_methods)
+        else:
+            raise ValueError("Unknown value: {}".format(now))
+
+    else:
+        now = datetime.datetime.utcnow()
+
+    return now
 
 
 def get_birthday(as_str=False, start_age=18, stop_age=100):
@@ -1953,10 +1380,7 @@ get_bday = get_birthday
 
 def get_past_datetime(now=None):
     """return a datetime guaranteed to be in the past from now"""
-    if not now: now = datetime.datetime.utcnow()
-    if isinstance(now, datetime.timedelta):
-        now = datetime.datetime.utcnow() - now
-
+    now = get_datetime(now, backward=True)
     td = now - datetime.datetime(year=2000, month=1, day=1)
     return now - datetime.timedelta(
         days=random.randint(1, max(td.days, 1)),
@@ -1975,10 +1399,7 @@ def get_past_date(now=None):
 
 def get_future_datetime(now=None):
     """return a datetime guaranteed to be in the future from now"""
-    if not now: now = datetime.datetime.utcnow()
-    if isinstance(now, datetime.timedelta):
-        now = datetime.datetime.utcnow() + now
-
+    now = get_datetime(now)
     return now + datetime.timedelta(
         weeks=random.randint(1, 52 * 50),
         hours=random.randint(0, 24),
@@ -2001,20 +1422,8 @@ def get_between_datetime(start, stop=None):
 
     return a datetime guaranteed to be in the future from start and in the past from stop
     """
-    if not stop:
-        stop = datetime.datetime.utcnow()
-
-    # account for start or stop being a timedelta
-    if isinstance(start, datetime.timedelta) and isinstance(stop, datetime.timedelta):
-        now = datetime.datetime.utcnow()
-        start = now - start
-        stop = now - stop
-
-    elif isinstance(start, datetime.timedelta):
-        start = stop - start
-
-    elif isinstance(stop, datetime.timedelta):
-        stop = start + stop
+    start = get_datetime(start)
+    stop = get_datetime(stop)
 
     if start >= stop:
         raise ValueError("start datetime >= stop datetime")
@@ -2041,53 +1450,6 @@ def get_between_date(start, stop=None):
     return datetime.date(dt.year, dt.month, dt.day)
 
 
-def get_interpreter():
-    """Return the best python interpreter
-
-    :returns: a Filepath
-    """
-
-    exe = sys.executable
-    version_info = sys.version_info
-
-    if re.search(r"\d+\.\d+", exe):
-        ret = exe
-
-    elif re.search(r"\d+", exe):
-        ret = "{}.{}".format(exe, version_info[1])
-        f = Filepath.get_instance(ret)
-        if not f.exists():
-            ret = exe
-
-    else:
-        # try major.minor
-        version = ".".join(map(String, version_info[0:2]))
-        ret = "{}{}".format(exe, version)
-        f = Filepath.get_instance(ret)
-
-        if not f.exists():
-            # try major
-            ret = "{}.{}".format(exe, version_info[1])
-            f = Filepath.get_instance(ret)
-            if not f.exists():
-                # just return the found interpreter
-                ret = exe
-
-    f = Filepath.get_instance(ret)
-
-    # https://semver.org/
-    f.major = version_info[0]
-    f.minor = version_info[1]
-    f.patch = version_info[2]
-    f.release = version_info[3]
-    f.version = "{}.{}.{}".format(f.major, f.minor, f.patch)
-
-    return f
-get_exe = get_interpreter
-get_exec = get_interpreter
-get_executable = get_interpreter
-get_python = get_interpreter
-
 # used in the get_unique_int() function to make sure it never returns the same int twice
 # this is a possible memory leak if you are using this script in a very long running
 # process using get_int(), since this list will get bigger and bigger and never
